@@ -52,18 +52,18 @@ AuthResultCode = enum(
     CREDENTIALS_REQUIRED=0,
     # OK results:
     GRANTED=1, # is also set when no authentication required
-    GRANTEDPASSWORDWILLEXPIRESOON = 2, # user should be notified about coming soon password expiry
-                                       # with suggestion to set a new password ASAP
+    GRANTED_PASSWORD_WILL_EXPIRE_SOON = 2, # user should be notified about coming soon password expiry
+                                           # with suggestion to set a new password ASAP
     # negative results:
-    NEWPASSWORDREQUIRED = 10, # password was OK but is expired, so new AuthRequest with additional
-                              # response with new password hash is required, and new password
-                              # complexity rules should be read from additionalCredentials[CredentialNewpassword].parameters
-    INVALIDCHALLENGERESPONSE = 11,
-    ADDITIONALRESPONSEREQUIRED = 12,
-    TEMPORARILYBLOCKED = 13,
-    REAUTHENTICATIONREQUIRED = 14 # server requires re-authentication (e.g.because of being idle),
-                                  # implementation should prompt the user for re-authentication
-                                  # (must not silently send cached credentials)
+    NEW_PASSWORD_REQUIRED = 10, # password was OK but is expired, so new AuthRequest with additional
+                                # response with new password hash is required, and new password
+                                # complexity rules should be read from additionalCredentials[CredentialNewpassword].parameters
+    INVALID_CHALLENGE_RESPONSE = 11,
+    ADDITIONAL_RESPONSE_REQUIRED = 12,
+    TEMPORARILY_BLOCKED = 13,
+    REAUTHENTICATION_REQUIRED = 14 # server requires re-authentication (e.g.because of being idle),
+                                   # implementation should prompt the user for re-authentication
+                                   # (must not silently send cached credentials)
 )
 
 class UserAuthResult:
@@ -218,16 +218,18 @@ class Node:
     def subscribe_to_structure_changes(self, callback):
         self._structure_subscriptions.append(callback)
 
-    def subscribe_to_value_changes(self, callback):
-        self._value_subscriptions.append(callback)
-        self._connection.send_value_request(self._id())
+    def subscribe_to_value_changes(self, callback, fs=5, sample_rate=0):
+        self._value_subscriptions.append((callback, fs, sample_rate))
+        self._send_value_request()
 
     def unsubscribe_from_structure_changes(self, callback):
         self._structure_subscriptions.remove(callback)
 
     def unsubscribe_from_value_changes(self, callback):
-        self._value_subscriptions.remove(callback)
-        if not self._value_subscriptions:
+        self._value_subscriptions = [i for i in self._value_subscriptions if i[0] != callback]
+        if self._value_subscriptions:
+            self._send_value_request()
+        else:
             self._connection.send_value_unrequest(self._id())
 
     def _id(self):
@@ -243,11 +245,16 @@ class Node:
 
         def fetch_value(node):
             if self._value_subscriptions:
-                self._connection.send_value_request(self._id())
+                self._send_value_request()
             return Promise(lambda resolve, reject: resolve(node))
 
         return fetch_structure().then(update_structure).then(fetch_value)
 
+    def _send_value_request(self):
+        max_fs = max(self._value_subscriptions, key=lambda e: e[1])[1]
+        max_sample_rate = max(self._value_subscriptions, key=lambda e: e[2])[2]
+        self._connection.send_value_request(self._id(), max_fs, max_sample_rate)
+
     def _update_structure(self, structure):
         self._structure = structure
         new_children = list(self._structure.node)
@@ -287,7 +294,7 @@ class Node:
 
     def _update_value(self, variant):
         self._value = self._value_from_variant(self._structure.info.value_type, variant)
-        for callback in self._value_subscriptions:
+        for callback, fs, sample_rate in self._value_subscriptions:
             callback(self._value, variant.timestamp + self._connection.server_time_difference() * nanoseconds_in_second)
 
     @staticmethod
@@ -416,13 +423,13 @@ class Connection:
             self._compose_and_send_structure_request(node_id)
         return p
 
-    def send_value_request(self, node_id):
+    def send_value_request(self, node_id, fs, sample_rate):
         self._update_time_difference()
-        self._compose_and_send_value_request(node_id)
+        self._compose_and_send_value_request(node_id, fs, sample_rate)
 
     def send_value_unrequest(self, node_id):
         self._update_time_difference()
-        self._compose_and_send_value_request(node_id, True)
+        self._compose_and_send_value_request(node_id, 1, 0, True)
 
     def send_value(self, variant):
         self._update_time_difference()
@@ -577,7 +584,7 @@ class Connection:
         if not self._re_auth_request:
             self._re_auth_request = AuthRequest(host=self._host, port=self._port, system_name=self._system_name,
                                                 application_name=self._application_name, cdp_version=self._cdp_version,
-                                                user_auth_result=UserAuthResult(code=AuthResultCode.REAUTHENTICATIONREQUIRED,
+                                                user_auth_result=UserAuthResult(code=AuthResultCode.REAUTHENTICATION_REQUIRED,
                                                                                 text=message))
             self._re_auth_request.then(self._re_authenticate)
             self._notification_listener.credentials_requested(self._re_auth_request)
@@ -593,7 +600,7 @@ class Connection:
             self._parse_structure_change_response(data.structure_change_response)
         elif data.message_type == proto.Container.eCurrentTimeResponse:
             self._parse_current_time_response(data.current_time_response)
-        elif data.message_type == proto.Container.eReAuthResponse:
+        elif data.message_type == proto.Container.eReauthResponse:
             self._parse_re_auth_response(data.re_auth_response)
         elif data.message_type == proto.Container.eRemoteError:
             self._parse_error_response(data.error)
@@ -638,7 +645,7 @@ class Connection:
     def _parse_hello_message(self, message):
         data = proto.Hello()
         data.ParseFromString(message)
-        if data.compat_version != 1:
+        if data.compat_version < 1:
             logging.info('Unsupported protocol version ' + str(data.compat_version) + '.' + str(data.incremental_version))
             return False
         self._system_name = data.system_name
@@ -677,12 +684,14 @@ class Connection:
             data.structure_request.append(node_id)
         self._ws.send(data.SerializeToString())
 
-    def _compose_and_send_value_request(self, node_id, stop=False):
+    def _compose_and_send_value_request(self, node_id, fs, sample_rate, stop=False):
         data = proto.Container()
         data.message_type = proto.Container.eGetterRequest
         value = proto.ValueRequest()
         value.node_id = node_id
-        value.fs = 5
+        value.fs = fs
+        if sample_rate is not 0:
+            value.sample_rate = sample_rate
         if stop:
             value.stop = stop
         data.getter_request.extend([value])
@@ -717,7 +726,7 @@ class Connection:
 
     def _compose_and_send_re_auth_request(self):
         container = proto.Container()
-        container.message_type = proto.Container.eReAuthRequest
+        container.message_type = proto.Container.eReauthRequest
         request = proto.AuthRequest()
         self._compose_auth_request(request)
         container.re_auth_request.CopyFrom(request)
