@@ -101,12 +101,13 @@ def create_error_response():
     return response
 
 
-def create_value_request(node_id):
+def create_value_request(node_id, fs, sample_rate):
     request = proto.Container()
     request.message_type = proto.Container.eGetterRequest
     value = proto.ValueRequest()
     value.node_id = node_id
-    value.fs = 5
+    value.fs = fs
+    value.sample_rate = sample_rate
     request.getter_request.extend([value])
     return request
 
@@ -116,7 +117,8 @@ def create_value_unrequest(node_id):
     request.message_type = proto.Container.eGetterRequest
     value = proto.ValueRequest()
     value.node_id = node_id
-    value.fs = 5
+    value.fs = 1
+    value.sample_rate = 0
     value.stop = True
     request.getter_request.extend([value])
     return request
@@ -156,7 +158,7 @@ def create_valid_hello_response_with_auth_required(challenge):
 def create_invalid_hello_response():
     response = proto.Hello()
     response.system_name = "foo"
-    response.compat_version = 2
+    response.compat_version = 0
     response.incremental_version = 0
     return response
 
@@ -201,7 +203,7 @@ def create_container_with_password_auth_request(challenge, user_id, password):
     user_pass_hash = sha256(user_id.encode().lower() + b':' + password.encode()).digest()
     response.response = sha256(challenge + b':' + user_pass_hash).digest()
     container = proto.Container()
-    container.message_type = proto.Container.eReAuthRequest
+    container.message_type = proto.Container.eReauthRequest
     container.re_auth_request.CopyFrom(request)
     return container
 
